
// ScribVw.h : interface of the CScribbleView class
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1997 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.
/////////////////////////////////////////////////////////////////////////////


// @@ added command's enumeration
enum
{
	CMD_NULL,

	CMD_ADDSTROKE,
	CMD_ADDSTROKES,
	CMD_REMOVESTROKE,
	CMD_REMOVESTROKES,
};

class CScribbleDoc;

// @@ added base command class to give each command a document pointer
class CCmdScribble : public CCommand
{
	DECLARE_COMMAND_NOID(CCmdScribble)

public:
	CCmdScribble(CScribbleDoc* pDoc)
		{ m_pDoc = pDoc; }
protected:
	CScribbleDoc* m_pDoc;
};

// @@ added command class
class CCmdAddStroke : public CCmdScribble
{
	DECLARE_COMMAND(CCmdAddStroke)

public:
	CCmdAddStroke(CScribbleDoc* pDoc, CStroke* pStroke) : CCmdScribble(pDoc)
		{ m_pStroke = pStroke; pStroke->AddRef(); }
	~CCmdAddStroke()
		{ m_pStroke->Release(); }

// Overrides
public:
	virtual BOOL Execute();
	virtual CCommand* GetUndoCommand() const;
	virtual CCommand* GetRedoCommand() const;
	virtual void Sprint(CString& strLabel) const
		{ strLabel = _T("Add Stroke"); }

// Attributes
protected:
	CStroke* m_pStroke;

	friend class CScribbleView;
};

// @@ added command class
class CCmdAddStrokes : public CCmdScribble
{
	DECLARE_COMMAND(CCmdAddStrokes)

public:
	CCmdAddStrokes(CScribbleDoc* pDoc, const CStrokeList& list);

// Overrides
public:
	virtual BOOL Execute();
	virtual CCommand* GetUndoCommand() const;
	virtual CCommand* GetRedoCommand() const;
	virtual void Sprint(CString& strLabel) const
		{ strLabel = _T("Add Strokes"); }

// Attributes
protected:
	CStrokeList m_list;

	friend class CScribbleView;
};

// @@ added command class
class CCmdRemoveStroke : public CCmdScribble
{
	DECLARE_COMMAND(CCmdRemoveStroke)

public:
	CCmdRemoveStroke(CScribbleDoc* pDoc, CStroke* pStroke) : CCmdScribble(pDoc)
		{ m_pStroke = pStroke; pStroke->AddRef(); }
	~CCmdRemoveStroke()
		{ m_pStroke->Release(); }

// Overrides
public:
	virtual BOOL Execute();
	virtual CCommand* GetUndoCommand() const;
	virtual CCommand* GetRedoCommand() const;
	virtual void Sprint(CString& strLabel) const
		{ strLabel = _T("Remove Stroke"); }

// Attributes
protected:
	CStroke* m_pStroke;

	friend class CScribbleView;
};

// @@ added command class
class CCmdRemoveStrokes : public CCmdScribble
{
	DECLARE_COMMAND(CCmdRemoveStrokes)

public:
	CCmdRemoveStrokes(CScribbleDoc* pDoc, const CStrokeList& list);

// Overrides
public:
	virtual BOOL Execute();
	virtual CCommand* GetUndoCommand() const;
	virtual CCommand* GetRedoCommand() const;
	virtual void Sprint(CString& strLabel) const
		{ strLabel = _T("Remove Strokes"); }

// Attributes
protected:
	CStrokeList m_list;

	friend class CScribbleView;
};


class CScribbleView : public CView
{
protected: // create from serialization only
	CScribbleView();
	DECLARE_DYNCREATE(CScribbleView)

// Attributes
public:
	CScribbleDoc* GetDocument();

protected:
	CStroke*    m_pStrokeCur;   // the stroke in progress
	CPoint      m_ptPrev;       // the last mouse pt in the stroke in progress

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CScribbleView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CScribbleView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CScribbleView)
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in ScribVw.cpp
inline CScribbleDoc* CScribbleView::GetDocument()
   { return (CScribbleDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////
